package me.wazup.hideandseek;

import java.math.RoundingMode;
import java.math.BigDecimal;
import org.bukkit.OfflinePlayer;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.Bukkit;
import me.wazup.hideandseek.utils.ItemStackBuilder;
import me.wazup.hideandseek.xseries.XMaterial;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.plugin.Plugin;
import org.bukkit.configuration.file.FileConfiguration;
import java.sql.ResultSet;
import java.sql.Statement;
import me.wazup.hideandseek.managers.FilesManager;
import java.sql.SQLException;
import org.bukkit.scheduler.BukkitRunnable;
import java.util.Iterator;
import org.bukkit.entity.Player;
import me.wazup.hideandseek.utils.FlexibleSmartInventory;
import java.util.HashMap;
import org.bukkit.Effect;
import org.bukkit.inventory.Inventory;
import java.util.ArrayList;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import java.util.Collection;
import org.bukkit.GameMode;
import org.bukkit.Location;

public class PlayerData
{
    public boolean played;
    public Location location;
    public GameMode gameMode;
    public boolean flying;
    public Collection<PotionEffect> effects;
    public Arena arena;
    public int hidersKilled;
    public int seekersKilled;
    public int wins;
    public int karma;
    public int gamesPlayed;
    public int hiderChance;
    public int modifier;
    public int playerExp;
    public Rank rank;
    double health;
    int food;
    int level;
    float exp;
    ItemStack[] inventory;
    ItemStack[] armor;
    Scoreboard scoreboard;
    int seconds;
    int minutes;
    int hours;
    int days;
    public ArrayList<ItemStack> specialBlocks;
    public ArrayList<Perk> perks;
    public Inventory specialBlocksShop;
    public Inventory hiderPerksShop;
    public Inventory seekerPerksShop;
    public Effect selectedTrail;
    public int currentGameCoins;
    public int currentGameExp;
    public int currentGameFireworks;
    public ArrayList<String> votes;
    private int coins;
    private final HashMap<String, Long> cooldowns;
    public FlexibleSmartInventory achievements;
    
    public PlayerData(final Player player) {
        this.specialBlocks = new ArrayList<ItemStack>();
        this.perks = new ArrayList<Perk>();
        this.currentGameCoins = 0;
        this.currentGameExp = 0;
        this.currentGameFireworks = 0;
        this.votes = new ArrayList<String>();
        this.cooldowns = new HashMap<String, Long>();
        this.loadStats(player);
    }
    
    public void saveData(final Player player, final Arena arena) {
        this.location = player.getLocation();
        this.health = player.getHealth();
        this.food = player.getFoodLevel();
        this.level = player.getLevel();
        this.exp = player.getExp();
        this.gameMode = player.getGameMode();
        this.flying = player.isFlying();
        this.effects = player.getActivePotionEffects();
        this.inventory = player.getInventory().getContents();
        this.armor = player.getInventory().getArmorContents();
        this.arena = arena;
        this.scoreboard = player.getScoreboard();
        this.played = true;
    }
    
    public void restoreData(final Player player) {
        player.teleport((Config.getInstance().TeleportToGlobalLobbyOnLeave && HideAndSeek.getInstance().globalLobby != null) ? HideAndSeek.getInstance().globalLobby : this.location);
        player.setHealth((this.health > player.getMaxHealth()) ? player.getMaxHealth() : this.health);
        player.setFoodLevel(this.food);
        player.setLevel(this.level);
        player.setExp(this.exp);
        player.setGameMode(this.gameMode);
        if (this.flying) {
            player.setAllowFlight(true);
            player.setFlying(true);
        }
        final Iterator iterator = player.getActivePotionEffects().iterator();
        while (iterator.hasNext()) {
            player.removePotionEffect(((PotionEffect)iterator.next()).getType());
        }
        player.addPotionEffects((Collection)this.effects);
        player.getInventory().setContents(this.inventory);
        player.getInventory().setArmorContents(this.armor);
        player.setScoreboard(this.scoreboard);
        this.destroyData();
    }
    
    public void destroyData() {
        this.arena = null;
        this.location = null;
        this.health = 0.0;
        this.food = 0;
        this.level = 0;
        this.exp = 0.0f;
        this.gameMode = null;
        this.effects = null;
        this.inventory = null;
        this.armor = null;
        this.scoreboard = null;
    }
    
    public void loadStats(final Player player) {
        new BukkitRunnable() {
            boolean statsExists = false;
            final /* synthetic */ String val$pName = Config.getInstance().useUUID ? player.getUniqueId().toString() : player.getName();
            
            public void run() {
                if (Config.getInstance().mysql_enabled) {
                    try {
                        final Statement statement = HideAndSeek.getInstance().mysql.getConnection().createStatement();
                        final ResultSet executeQuery = statement.executeQuery("select * from " + HideAndSeek.getInstance().mysql.tableName + " where " + (Config.getInstance().useUUID ? "player_uuid" : "player_name") + " = '" + this.val$pName + "';");
                        if (executeQuery.next()) {
                            PlayerData.this.hidersKilled = executeQuery.getInt("hiders_killed");
                            PlayerData.this.seekersKilled = executeQuery.getInt("seekers_killed");
                            PlayerData.this.wins = executeQuery.getInt("wins");
                            PlayerData.this.coins = executeQuery.getInt("coins");
                            PlayerData.this.karma = executeQuery.getInt("karma");
                            PlayerData.this.gamesPlayed = executeQuery.getInt("games_played");
                            PlayerData.this.hiderChance = executeQuery.getInt("hider_chance");
                            final String string = executeQuery.getString("play_time");
                            PlayerData.this.days = Integer.parseInt(string.split("-")[0].replace("D", ""));
                            PlayerData.this.hours = Integer.parseInt(string.split("-")[1].replace("H", ""));
                            PlayerData.this.minutes = Integer.parseInt(string.split("-")[2].replace("M", ""));
                            PlayerData.this.seconds = Integer.parseInt(string.split("-")[3].replace("S", ""));
                            PlayerData.this.playerExp = executeQuery.getInt("exp");
                            PlayerData.this.modifier = executeQuery.getInt("modifier");
                            for (final Rank rank : Rank.ranks) {
                                if (PlayerData.this.playerExp < rank.requiredExp) {
                                    break;
                                }
                                PlayerData.this.rank = rank;
                            }
                            try {
                                PlayerData.this.loadSelectedTrail(executeQuery.getInt("trail"));
                            }
                            catch (final Exception ex) {}
                            this.statsExists = true;
                            PlayerData.this.readSpecialBlocksString(executeQuery.getString("special_blocks"));
                            PlayerData.this.loadSpecialBlocksShop();
                            PlayerData.this.loadPerks(executeQuery.getString("perks"));
                            PlayerData.this.loadPerksShop();
                        }
                        statement.close();
                        executeQuery.close();
                    }
                    catch (final SQLException ex2) {
                        ex2.printStackTrace();
                    }
                }
                else {
                    final FileConfiguration config = FilesManager.getInstance().getConfig("players.yml");
                    final String string2 = "Players." + this.val$pName;
                    if (config.contains(string2)) {
                        PlayerData.this.hidersKilled = config.getInt(string2 + ".hidersKilled");
                        PlayerData.this.seekersKilled = config.getInt(string2 + ".seekersKilled");
                        PlayerData.this.wins = config.getInt(string2 + ".wins");
                        PlayerData.this.coins = config.getInt(string2 + ".coins");
                        PlayerData.this.karma = config.getInt(string2 + ".karma");
                        PlayerData.this.gamesPlayed = config.getInt(string2 + ".gamesPlayed");
                        PlayerData.this.hiderChance = config.getInt(string2 + ".hiderChance");
                        final String string3 = config.getString(string2 + ".playTime");
                        PlayerData.this.days = Integer.valueOf(string3.split("-")[0].replace("D", ""));
                        PlayerData.this.hours = Integer.valueOf(string3.split("-")[1].replace("H", ""));
                        PlayerData.this.minutes = Integer.valueOf(string3.split("-")[2].replace("M", ""));
                        PlayerData.this.seconds = Integer.valueOf(string3.split("-")[3].replace("S", ""));
                        PlayerData.this.playerExp = config.getInt(string2 + ".Exp");
                        PlayerData.this.modifier = config.getInt(string2 + ".Modifier");
                        for (final Rank rank2 : Rank.ranks) {
                            if (PlayerData.this.playerExp < rank2.requiredExp) {
                                break;
                            }
                            PlayerData.this.rank = rank2;
                        }
                        if (config.contains(string2 + ".trail")) {
                            PlayerData.this.loadSelectedTrail(config.getInt(string2 + ".trail"));
                        }
                        PlayerData.this.readSpecialBlocksString(config.getString(string2 + ".Special-Blocks"));
                        PlayerData.this.loadSpecialBlocksShop();
                        PlayerData.this.loadPerks(config.getString(string2 + ".Perks"));
                        PlayerData.this.loadPerksShop();
                        this.statsExists = true;
                    }
                }
                if (!this.statsExists) {
                    PlayerData.this.hidersKilled = 0;
                    PlayerData.this.seekersKilled = 0;
                    PlayerData.this.wins = 0;
                    PlayerData.this.coins = 0;
                    PlayerData.this.karma = 0;
                    PlayerData.this.gamesPlayed = 0;
                    PlayerData.this.hiderChance = Config.getInstance().StartingHiderChance;
                    PlayerData.this.seconds = 0;
                    PlayerData.this.minutes = 0;
                    PlayerData.this.hours = 0;
                    PlayerData.this.days = 0;
                    PlayerData.this.playerExp = 0;
                    PlayerData.this.modifier = 1;
                    PlayerData.this.loadSpecialBlocksShop();
                    PlayerData.this.loadPerksShop();
                    PlayerData.this.rank = Rank.ranks.get(0);
                }
            }
        }.runTaskAsynchronously((Plugin)HideAndSeek.getInstance());
    }
    
    public void saveStats(final Player player, final boolean b) {
        if (!this.played) {
            return;
        }
        if (b) {
            new BukkitRunnable() {
                public void run() {
                    if (Config.getInstance().mysql_enabled) {
                        PlayerData.this.saveStatsIntoMySQL(player);
                    }
                    else {
                        PlayerData.this.saveStatsToFile(player);
                    }
                }
            }.runTaskAsynchronously((Plugin)HideAndSeek.getInstance());
        }
        else if (Config.getInstance().mysql_enabled) {
            this.saveStatsIntoMySQL(player);
        }
        else {
            this.saveStatsToFile(player);
        }
    }
    
    private void saveStatsIntoMySQL(final Player player) {
        final String s = Config.getInstance().useUUID ? player.getUniqueId().toString() : player.getName();
        try {
            final Statement statement = HideAndSeek.getInstance().mysql.getConnection().createStatement();
            if (statement.executeQuery("select * from " + HideAndSeek.getInstance().mysql.tableName + " where " + (Config.getInstance().useUUID ? "player_uuid" : "player_name") + " = '" + s + "'").next()) {
                HideAndSeek.getInstance().mysql.getConnection().prepareStatement("update " + HideAndSeek.getInstance().mysql.tableName + " set hiders_killed=" + this.hidersKilled + ", seekers_killed=" + this.seekersKilled + ", wins=" + this.wins + ", coins=" + this.coins + ", karma=" + this.karma + ", games_played=" + this.gamesPlayed + ", hider_chance=" + this.hiderChance + ", play_time='" + this.getPlayTime(player) + "', exp=" + this.playerExp + ", modifier=" + this.modifier + ", special_blocks='" + this.getSpecialBlocksString() + "', perks='" + this.getPerksString() + "', trail=" + this.getSelectedTrailSlot(player) + " where " + (Config.getInstance().useUUID ? "player_uuid" : "player_name") + " = '" + s + "';").executeUpdate();
            }
            else {
                statement.executeUpdate("insert into " + HideAndSeek.getInstance().mysql.tableName + " (player_name, player_uuid, hiders_killed, seekers_killed, wins, coins, karma, games_played, hider_chance, play_time, exp, modifier, special_blocks, perks, trail) values ('" + player.getName() + "', '" + player.getUniqueId() + "', " + this.hidersKilled + ", " + this.seekersKilled + ", " + this.wins + ", " + this.coins + ", " + this.karma + ", " + this.gamesPlayed + ", " + this.hiderChance + ", '" + this.getPlayTime(player) + "', " + this.playerExp + ", " + this.modifier + ", '" + this.getSpecialBlocksString() + "', '" + this.getPerksString() + "', " + this.getSelectedTrailSlot(player) + ")");
            }
            statement.close();
        }
        catch (final SQLException ex) {
            ex.printStackTrace();
        }
    }
    
    private void saveStatsToFile(final Player player) {
        final String str = Config.getInstance().useUUID ? player.getUniqueId().toString() : player.getName();
        final FileConfiguration config = FilesManager.getInstance().getConfig("players.yml");
        final String string = "Players." + str;
        config.set(string + ".Name", (Object)player.getName());
        config.set(string + ".hidersKilled", (Object)this.hidersKilled);
        config.set(string + ".seekersKilled", (Object)this.seekersKilled);
        config.set(string + ".wins", (Object)this.wins);
        config.set(string + ".coins", (Object)this.coins);
        config.set(string + ".karma", (Object)this.karma);
        config.set(string + ".gamesPlayed", (Object)this.gamesPlayed);
        config.set(string + ".hiderChance", (Object)this.hiderChance);
        config.set(string + ".playTime", (Object)this.getPlayTime(player));
        config.set(string + ".Exp", (Object)this.playerExp);
        config.set(string + ".Modifier", (Object)this.modifier);
        config.set(string + ".Special-Blocks", (Object)this.getSpecialBlocksString());
        config.set(string + ".Perks", (Object)this.getPerksString());
        config.set(string + ".trail", (Object)this.getSelectedTrailSlot(player));
        FilesManager.getInstance().saveConfig("players.yml");
    }
    
    public String getPlayTime(final Player player) {
        return this.days + "D-" + this.hours + "H-" + this.minutes + "M-" + this.seconds + "S";
    }
    
    private void loadSelectedTrail(final int n) {
        if (HideAndSeek.getInstance().trailsInventory != null && n >= 0 && n < HideAndSeek.getInstance().trailsInventory.getSize()) {
            final ItemStack item = HideAndSeek.getInstance().trailsInventory.getItem(n);
            if (item != null && !item.getType().equals((Object)Material.AIR)) {
                this.selectedTrail = Effect.valueOf(ChatColor.stripColor(item.getItemMeta().getDisplayName()));
            }
        }
    }
    
    private int getSelectedTrailSlot(final Player player) {
        if (this.selectedTrail == null || !player.hasPermission("hideandseek.trails")) {
            return -1;
        }
        for (int i = 0; i < HideAndSeek.getInstance().trailsInventory.getSize(); ++i) {
            if (HideAndSeek.getInstance().trailsInventory.getItem(i) != null && ChatColor.stripColor(HideAndSeek.getInstance().trailsInventory.getItem(i).getItemMeta().getDisplayName()).equals(this.selectedTrail.name())) {
                return i;
            }
        }
        return -1;
    }
    
    public ArrayList<String> getStats(final Player player) {
        final ArrayList list = new ArrayList();
        list.add(ChatColor.AQUA + "" + ChatColor.STRIKETHROUGH + "---------" + ChatColor.GOLD + " " + player.getName() + " " + ChatColor.AQUA + "" + ChatColor.STRIKETHROUGH + "---------");
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Hiders killed" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.hidersKilled);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Seekers killed" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.seekersKilled);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Wins" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.wins);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Coins" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.getCoins(player));
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Karma" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.karma);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Games played" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.gamesPlayed);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Hider chance" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.hiderChance + "%");
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Play time" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.days + "D" + ChatColor.GOLD + "-" + ChatColor.GREEN + this.hours + "H " + ChatColor.GOLD + "-" + ChatColor.GREEN + this.minutes + "M" + ChatColor.GOLD + "-" + ChatColor.GREEN + this.seconds + "S");
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Rank" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.rank.name);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Exp" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.playerExp);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Next rank" + ChatColor.GOLD + ": " + ChatColor.GREEN + ((this.rank.nextRank != null) ? (this.rank.nextRank.requiredExp - this.playerExp) : 0));
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Modifier" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.modifier);
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Perks" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.perks.size());
        list.add(ChatColor.AQUA + " - " + ChatColor.YELLOW + "Special blocks" + ChatColor.GOLD + ": " + ChatColor.GREEN + this.specialBlocks.size());
        list.add(ChatColor.AQUA + "" + ChatColor.STRIKETHROUGH + "---------------------------");
        return list;
    }
    
    protected String getSpecialBlocksString() {
        if (this.specialBlocks.isEmpty()) {
            return "";
        }
        String str = this.specialBlocks.get(0).getType().name() + ":" + this.specialBlocks.get(0).getDurability();
        for (int i = 1; i < this.specialBlocks.size(); ++i) {
            str = str + ", " + this.specialBlocks.get(i).getType().name() + ":" + this.specialBlocks.get(i).getDurability();
        }
        return str;
    }
    
    protected String getPerksString() {
        if (this.perks.isEmpty()) {
            return "";
        }
        String str = this.perks.get(0).name;
        for (int i = 1; i < this.perks.size(); ++i) {
            str = str + ", " + this.perks.get(i).name;
        }
        return str;
    }
    
    protected void readSpecialBlocksString(final String s) {
        if (s == null || s.isEmpty()) {
            return;
        }
        if (s.contains(", ")) {
            final String[] split = s.split(", ");
            for (int i = 0; i < split.length; ++i) {
                this.specialBlocks.add(new ItemStackBuilder(((XMaterial)XMaterial.matchXMaterial(split[i]).get()).parseItem()).setName(ChatColor.GOLD + "Special block").build());
            }
        }
        else {
            this.specialBlocks.add(new ItemStackBuilder(XMaterial.matchXMaterial(s).get().parseItem()).setName(ChatColor.GOLD + "Special block").build());
        }
    }
    
    protected void loadSpecialBlocksShop() {
        this.specialBlocksShop = Bukkit.createInventory((InventoryHolder)null, 54, (String)HideAndSeek.getInstance().customization.inventories.get("Special-Blocks"));
        for (int i = 0; i < 9; ++i) {
            this.specialBlocksShop.setItem(i, HideAndSeek.getInstance().locked);
        }
        for (int j = 45; j < 54; ++j) {
            this.specialBlocksShop.setItem(j, HideAndSeek.getInstance().locked);
        }
        for (int k = 9; k < 45; k += 9) {
            this.specialBlocksShop.setItem(k, HideAndSeek.getInstance().locked);
        }
        for (int l = 17; l < 53; l += 9) {
            this.specialBlocksShop.setItem(l, HideAndSeek.getInstance().locked);
        }
        this.specialBlocksShop.setItem(49, (ItemStack)Customization.getInstance().items.get("Back"));
        for (final ItemStack itemStack : Config.getInstance().shopSpecial.keySet()) {
            if (!this.specialBlocks.contains(itemStack)) {
                this.specialBlocksShop.addItem(new ItemStack[] { new ItemStackBuilder(itemStack.clone()).addLore(new String[] { ChatColor.GOLD + "Price: " + ChatColor.YELLOW + (int)Config.getInstance().shopSpecial.get(itemStack) }).build() });
            }
        }
    }
    
    protected void loadPerks(final String anotherString) {
        if (anotherString != null && !anotherString.isEmpty()) {
            if (anotherString.contains(", ")) {
                final String[] split = anotherString.split(", ");
                for (int i = 0; i < split.length; ++i) {
                    final String anotherString2 = split[i];
                    for (final Perk e : Perk.values()) {
                        if (e.name.equalsIgnoreCase(anotherString2)) {
                            this.perks.add(e);
                            break;
                        }
                    }
                }
            }
            else {
                for (final Perk e2 : Perk.values()) {
                    if (e2.name.equalsIgnoreCase(anotherString)) {
                        this.perks.add(e2);
                        break;
                    }
                }
            }
        }
    }
    
    protected void loadPerksShop() {
        this.hiderPerksShop = Bukkit.createInventory((InventoryHolder)null, 45, (String)HideAndSeek.getInstance().customization.inventories.get("Perks"));
        this.seekerPerksShop = Bukkit.createInventory((InventoryHolder)null, 45, (String)HideAndSeek.getInstance().customization.inventories.get("Perks"));
        for (int i = 0; i < 45; ++i) {
            this.hiderPerksShop.setItem(i, HideAndSeek.getInstance().locked);
            this.seekerPerksShop.setItem(i, HideAndSeek.getInstance().locked);
        }
        this.hiderPerksShop.setItem(40, (ItemStack)Customization.getInstance().items.get("Back"));
        this.seekerPerksShop.setItem(40, (ItemStack)Customization.getInstance().items.get("Back"));
        this.hiderPerksShop.setItem(19, this.hasPerk(Perk.DisguiseSelector) ? Perk.DisguiseSelector.purchased : (Perk.DisguiseSelector.enabled ? Perk.DisguiseSelector.unpurchased : Perk.DisguiseSelector.disabled));
        this.hiderPerksShop.setItem(22, this.hasPerk(Perk.DisguiseChanger) ? Perk.DisguiseChanger.purchased : (Perk.DisguiseChanger.enabled ? Perk.DisguiseChanger.unpurchased : Perk.DisguiseChanger.disabled));
        this.hiderPerksShop.setItem(25, this.hasPerk(Perk.InstaSolid) ? Perk.InstaSolid.purchased : (Perk.InstaSolid.enabled ? Perk.InstaSolid.unpurchased : Perk.InstaSolid.disabled));
        if (HideAndSeek.getInstance().disguiseManager != null) {
            this.seekerPerksShop.setItem(19, this.hasPerk(Perk.DisguisesDetector) ? Perk.DisguisesDetector.purchased : (Perk.DisguisesDetector.enabled ? Perk.DisguisesDetector.unpurchased : Perk.DisguisesDetector.disabled));
            this.seekerPerksShop.setItem(22, this.hasPerk(Perk.SeekerSpeed) ? Perk.SeekerSpeed.purchased : (Perk.SeekerSpeed.enabled ? Perk.SeekerSpeed.unpurchased : Perk.SeekerSpeed.disabled));
            this.seekerPerksShop.setItem(25, this.hasPerk(Perk.ThrowableTNT) ? Perk.ThrowableTNT.purchased : (Perk.ThrowableTNT.enabled ? Perk.ThrowableTNT.unpurchased : Perk.ThrowableTNT.disabled));
        }
        else {
            this.seekerPerksShop.setItem(19, this.hasPerk(Perk.SeekerSpeed) ? Perk.SeekerSpeed.purchased : (Perk.SeekerSpeed.enabled ? Perk.SeekerSpeed.unpurchased : Perk.SeekerSpeed.disabled));
            this.seekerPerksShop.setItem(25, this.hasPerk(Perk.ThrowableTNT) ? Perk.ThrowableTNT.purchased : (Perk.ThrowableTNT.enabled ? Perk.ThrowableTNT.unpurchased : Perk.ThrowableTNT.disabled));
        }
    }
    
    public boolean hasCoins(final Player player, final int n) {
        return this.getCoins(player) >= n;
    }
    
    public void addCoins(final Player player, final int n) {
        if (HideAndSeek.getInstance().econ != null) {
            HideAndSeek.getInstance().econ.depositPlayer(player.getName(), (double)n);
        }
        else {
            this.coins += n;
        }
    }
    
    public void removeCoins(final Player player, final int n) {
        if (HideAndSeek.getInstance().econ != null) {
            HideAndSeek.getInstance().econ.withdrawPlayer(player.getName(), (double)n);
        }
        else {
            this.coins -= n;
        }
    }
    
    public void setCoins(final Player player, final int coins) {
        if (HideAndSeek.getInstance().econ != null) {
            final int n = coins - this.getCoins(player);
            if (n > 0) {
                HideAndSeek.getInstance().econ.depositPlayer((OfflinePlayer)player, (double)n);
            }
            else {
                HideAndSeek.getInstance().econ.withdrawPlayer((OfflinePlayer)player, (double)(-n));
            }
        }
        else {
            this.coins = coins;
        }
    }
    
    public int getCoins(final Player player) {
        if (HideAndSeek.getInstance().econ != null) {
            return (int)HideAndSeek.getInstance().econ.getBalance((OfflinePlayer)player);
        }
        return this.coins;
    }
    
    public boolean hasPerk(final Perk o) {
        return o.enabled && this.perks.contains(o);
    }
    
    public void reset(final Player player) {
        this.hidersKilled = 0;
        this.seekersKilled = 0;
        this.wins = 0;
        if (player != null) {
            this.setCoins(player, 0);
        }
        this.karma = 0;
        this.gamesPlayed = 0;
        this.hiderChance = Config.getInstance().StartingHiderChance;
        this.modifier = 1;
        this.seconds = 0;
        this.minutes = 0;
        this.hours = 0;
        this.days = 0;
        this.specialBlocks.clear();
        this.perks.clear();
        this.playerExp = 0;
        this.rank = Rank.ranks.get(0);
        this.loadSpecialBlocksShop();
        this.loadPerksShop();
    }
    
    public boolean hasCooldown(final Player player, final String key, final int n, final boolean b) {
        final long cooldown = this.getCooldown(key);
        if (cooldown > 0L) {
            if (b) {
                player.sendMessage(HideAndSeek.getInstance().customization.messages.get("cooldown").replace("%time%", String.valueOf(new BigDecimal(cooldown / 1000.0).setScale(1, RoundingMode.HALF_UP).doubleValue())));
            }
            return true;
        }
        this.cooldowns.put(key, System.currentTimeMillis() + n * 1000);
        return false;
    }
    
    public long getCooldown(final String s) {
        return this.cooldowns.containsKey(s) ? (this.cooldowns.get(s) - System.currentTimeMillis()) : 0L;
    }
}
